#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>
#include <avr/eeprom.h>
#include <stdlib.h>
#include <portbits.h>
#include "global.h"
#include "uart.h"
#include "rf12.h"
#include "leds.h"

#define F_CPU 10000000UL
#include <util/delay.h>

#define RF_BAUDRATE		4800
#define UART_BAUDRATE	19200

#define MAX_BUF			128

volatile unsigned char delaycnt;
unsigned char rxbuf[MAX_BUF];
unsigned char txbuf[MAX_BUF];

int main(void)
{
	DDRD=255;
	DDRC=255;
    uart_init(UART_BAUD_SELECT(UART_BAUDRATE, F_CPU));
	rf12_init();					// ein paar Register setzen (z.B. CLK auf 10MHz)
	rf12_setfreq(RF12FREQ(433.92));	// Sende/Empfangsfrequenz auf 433,92MHz einstellen
	rf12_setbandwidth(4, 0, 1);		// 200kHz Bandbreite, -6dB Verstrkung, DRSSI threshold: -97dBm 
	rf12_setbaud(RF_BAUDRATE);		// Baudrate
	rf12_setpower(0, 6);			// 6mW Ausgangangsleistung, 120kHz Frequenzshift

	TCCR1A=0;
	TCCR1B=(1<<WGM12)|1;
	OCR1A=((F_CPU+500)/1000)-1;
	TIMSK=(1<<OCIE1A);

	sei();
	unsigned char tx_cnt=0,rx_cnt,i;

	rf12_rxmode();

	for (;;)
	{	if (rf12_data())					// Daten im RF12 RX Puffer ?
		{	rx_cnt=rf12_rxdata(rxbuf);		// J< -> komplettes Paket empfangen
			if (rx_cnt==255)				// Checksum error
			{	LED_OK=0;
			}
			else
			{	LED_OK=1;
				for (i=0; i<rx_cnt; i++)
					uart_putc(rxbuf[i]);
			}
		}
		if (uart_data())					// Daten im UART RX Puffer ?
		{	if (!tx_cnt)
				delaycnt=100;				// Nach dem ersten Byte: 100ms timeout starten
			txbuf[tx_cnt++]=uart_getchar();
		}
		if (((tx_cnt)&&(delaycnt==0))||(tx_cnt>=MAX_BUF))	// Puffer voll, oder 100ms seit erstem Byte im Puffer vorbei ?  -> senden
		{	rf12_stoprx();					// auf TX umschalten			
			_delay_ms(1);
			rf12_txdata(txbuf, tx_cnt);		// senden
			tx_cnt=0;
			rf12_rxmode();					// wieder auf RX umschalten
		}
	}
}

ISR(SIG_OUTPUT_COMPARE1A)					// 1kHz Interrupt
{	
	if (delaycnt)
		delaycnt--;
}



