#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>
#include <portbits.h>
#include "global.h"
#include "rf12.h"
#include "leds.h"

#define F_CPU 10000000UL
#include <util/delay.h>

#define RF_PORT	PORTB
#define RF_DDR	DDRB
#define RF_PIN	PINB

#define SDI		3
#define SCK		5
#define CS		2
#define SDO		4

#define MAX_BUF		128


prog_uint8_t hamminge[16]={0x15,0x02,0x49,0x5E,0x64,0x73,0x38,0x2F,0xD0,0xC7,0x8C,0x9B,0xA1,0xB6,0xFD,0xEA};
prog_uint8_t hammingd[256]={
0x01,0x00,0x01,0x01,0x00,0x00,0x01,0x01,0x02,0x02,0x01,0x03,0x0A,0x02,0x03,0x07,
0x00,0x00,0x01,0x01,0x00,0x00,0x01,0x00,0x06,0x02,0x03,0x0B,0x02,0x00,0x03,0x03,
0x04,0x0C,0x01,0x05,0x04,0x04,0x05,0x07,0x06,0x06,0x07,0x07,0x06,0x07,0x07,0x07,
0x06,0x04,0x05,0x05,0x04,0x00,0x0D,0x05,0x06,0x06,0x06,0x07,0x06,0x06,0x07,0x07,
0x00,0x02,0x01,0x01,0x04,0x00,0x01,0x09,0x02,0x02,0x03,0x02,0x02,0x02,0x03,0x03,
0x08,0x00,0x01,0x05,0x00,0x00,0x03,0x01,0x02,0x02,0x03,0x03,0x03,0x02,0x03,0x03,
0x04,0x04,0x05,0x05,0x04,0x04,0x04,0x05,0x06,0x02,0x0F,0x07,0x04,0x06,0x07,0x07,
0x04,0x05,0x05,0x05,0x04,0x04,0x05,0x05,0x06,0x06,0x07,0x05,0x06,0x0E,0x03,0x07,
0x08,0x0C,0x01,0x09,0x0A,0x08,0x09,0x09,0x0A,0x0A,0x0B,0x0B,0x0A,0x0A,0x0A,0x0B,
0x08,0x08,0x09,0x0B,0x08,0x00,0x0D,0x09,0x0A,0x0B,0x0B,0x0B,0x0A,0x0A,0x0B,0x0B,
0x0C,0x0C,0x0D,0x0C,0x0C,0x0C,0x0D,0x0D,0x0E,0x0C,0x0F,0x0F,0x0A,0x0E,0x0F,0x07,
0x0C,0x0C,0x0D,0x0D,0x0D,0x0C,0x0D,0x0D,0x06,0x0E,0x0F,0x0B,0x0E,0x0E,0x0D,0x0F,
0x08,0x08,0x09,0x09,0x08,0x09,0x09,0x09,0x0A,0x02,0x0F,0x0B,0x0A,0x0A,0x0B,0x09,
0x08,0x08,0x08,0x09,0x08,0x08,0x09,0x09,0x08,0x0A,0x0B,0x0B,0x0A,0x0E,0x03,0x0B,
0x0C,0x0C,0x0F,0x0D,0x04,0x0C,0x0D,0x09,0x0F,0x0E,0x0F,0x0F,0x0E,0x0E,0x0F,0x0F,
0x08,0x0C,0x0D,0x05,0x0C,0x0E,0x0D,0x0D,0x0E,0x0E,0x0F,0x0F,0x0E,0x0E,0x0F,0x0E};

unsigned short rf12_trans(unsigned short wert)
{	CONVERTW val;
	val.w=wert;
	cbi(RF_PORT, CS);
	SPDR = val.b[1];
	while(!(SPSR & (1<<SPIF)));
	val.b[1]=SPDR;
	SPDR = val.b[0];
	while(!(SPSR & (1<<SPIF)));
	val.b[0]=SPDR;
	sbi(RF_PORT, CS);
	return val.w;
}

void rf12_init(void)
{
	RF_DDR=(1<<SDI)|(1<<SCK)|(1<<CS);
	RF_PORT=(1<<CS);
	SPCR=(1<<SPE)|(1<<MSTR);

	for (unsigned char i=0; i<20; i++)
		_delay_ms(10);			// wait until POR done
	rf12_trans(0xC0E0);			// AVR CLK: 10MHz
	rf12_trans(0x80D7);			// Enable FIFO
	rf12_trans(0xC2AB);			// Data Filter: internal
	rf12_trans(0xCA81);			// Set FIFO mode
	rf12_trans(0xE000);			// disable wakeuptimer
	rf12_trans(0xC800);			// disable low duty cycle
	rf12_trans(0xC4F7);			// AFC settings: autotuning: -10kHz...+7,5kHz
}

void rf12_rxmode(void)
{
	LED_RX=1;
	rf12_trans(0x82C8);				// RX on
	_delay_ms(1);
	rf12_trans(0xCA81);				// set FIFO mode
	rf12_trans(0xCA83);				// enable FIFO: sync word search
}

void rf12_stoprx(void)
{
	rf12_trans(0x8208);			// RX off
	LED_RX=0;
}

void rf12_setbandwidth(unsigned char bandwidth, unsigned char gain, unsigned char drssi)
{
	rf12_trans(0x9400|((bandwidth&7)<<5)|((gain&3)<<3)|(drssi&7));
}

void rf12_setfreq(unsigned short freq)
{	if (freq<96)				// 430,2400MHz
		freq=96;
	else if (freq>3903)			// 439,7575MHz
		freq=3903;
	rf12_trans(0xA000|freq);
}

void rf12_setbaud(unsigned short baud)
{
	if (baud<664)
		baud=664;
	if (baud<5400)					// Baudrate= 344827,58621/(R+1)/(1+CS*7)
		rf12_trans(0xC680|((43104/baud)-1));	// R=(344828/8)/Baud-1
	else
		rf12_trans(0xC600|((344828UL/baud)-1));	// R=344828/Baud-1
}

void rf12_setpower(unsigned char power, unsigned char mod)
{	
	rf12_trans(0x9800|(power&7)|((mod&15)<<4));
}

void rf12_ready(void)
{
	cbi(RF_PORT, CS);
	cbi(RF_PORT, SDI);
	asm("nop");
	while (!(RF_PIN&(1<<SDO))); // wait until FIFO ready
}

unsigned rf12_data(void)
{
	cbi(RF_PORT, CS);
	cbi(RF_PORT, SDI);
	asm("nop");
	if(RF_PIN&(1<<SDO))
		return 1;
	else
		return 0;
}

void rf12_txdata(unsigned char *data, unsigned char number)
{	unsigned char i, checksum;
	LED_TX=1;
	LED_ACT=1;
	rf12_trans(0x8238);			// TX on
	rf12_ready();
	rf12_trans(0xB8AA);
	rf12_ready();
	rf12_trans(0xB8AA);
	rf12_ready();
	rf12_trans(0xB8AA);
	rf12_ready();
	rf12_trans(0xB82D);
	rf12_ready();
	rf12_trans(0xB8D4);
	rf12_ready();
	rf12_trans(0xB800|(pgm_read_byte(&hamminge[number/16])));	// Anzahl der zu senden Bytes bertragen
	rf12_ready();
	rf12_trans(0xB800|(pgm_read_byte(&hamminge[number&15])));
	checksum=0;
	for (i=0; i<number; i++)
	{	rf12_ready();
		rf12_trans(0xB800|(pgm_read_byte(&hamminge[*data/16])));
		rf12_ready();
		rf12_trans(0xB800|(pgm_read_byte(&hamminge[*data&15])));
		checksum+=*data;
		data++;
	}
	checksum=256-checksum;
	rf12_ready();
	rf12_trans(0xB800|(pgm_read_byte(&hamminge[checksum/16])));	// Checksumme hinterher senden
	rf12_ready();
	rf12_trans(0xB800|(pgm_read_byte(&hamminge[checksum&15])));
	rf12_ready();
	rf12_trans(0xB800);			// dummy data
	rf12_ready();
	rf12_trans(0x8208);			// TX off
	LED_ACT=0;
	LED_TX=0;
}

unsigned char rf12_rxdata(unsigned char *data)
{	unsigned char i, number,checksum;
	rf12_ready();
	LED_ACT=1;
	number =(pgm_read_byte(&hammingd[rf12_trans(0xB000)&255])*16);	// Anzahl der zu empfangenden Bytes senden
	rf12_ready();
	number|=(pgm_read_byte(&hammingd[rf12_trans(0xB000)&255]));
	if (number>MAX_BUF)
		number=MAX_BUF;
	checksum=0;
	for (i=0; i<number; i++)
	{	rf12_ready();
		*data =(pgm_read_byte(&hammingd[rf12_trans(0xB000)&255])*16);
		rf12_ready();
		*data|=(pgm_read_byte(&hammingd[rf12_trans(0xB000)&255]));
		checksum+=*data;
		data++;
	}
	rf12_ready();
	i=(pgm_read_byte(&hammingd[rf12_trans(0xB000)&255])*16);		// Checksumme empfangen
	rf12_ready();
	i|=(pgm_read_byte(&hammingd[rf12_trans(0xB000)&255]));
	checksum+=i;
	LED_ACT=0;
	rf12_trans(0xCA81);			// restart syncword detection:
	rf12_trans(0xCA83);			// enable FIFO
	if (checksum)
		return 255;				// checksum error
	else
		return number;			// data ok
}

